local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
if not HeadCount then return end

local L = LibStub("AceLocale-3.0"):GetLocale("HeadCount2")

HeadCount.DateFormats = {
	[1] = L["Day-Month-Year"],
	[2] = L["Month-Day-Year"],
	[3] = L["Year-Day-Month"],
}

local partyGroups = { "1", "2", "3", "4", "5", "6", "7", "8", }
local exportFormats = {
	["EQdkp"] = L["EQdkp"],
	["DKPBoard"] = L["DKPBoard"],
	["phpBB"] = L["phpBB"],
	["phpBB_ItemStats"] = L["phpBB_ItemStats"],
	["CSV"] = L["CSV"],
	["Text"] = L["Text"],
	["XML"] = L["XML"],
}

local channels = {
	["GUILD"] = CHAT_MSG_GUILD,
	["OFFICER"] = CHAT_MSG_OFFICER,
	["PARTY"] = CHAT_MSG_PARTY,
	["RAID"] = CHAT_MSG_RAID,
	["SAY"] = CHAT_MSG_SAY,
	["YELL"] = CHAT_MSG_YELL,
}

local lootQualityList = {
	[ITEM_QUALITY_COMMON] = ITEM_QUALITY_COLORS[ITEM_QUALITY_COMMON].hex..ITEM_QUALITY1_DESC.."|r",
	[ITEM_QUALITY_UNCOMMON] = ITEM_QUALITY_COLORS[ITEM_QUALITY_UNCOMMON].hex..ITEM_QUALITY2_DESC.."|r",
	[ITEM_QUALITY_RARE] = ITEM_QUALITY_COLORS[ITEM_QUALITY_RARE].hex..ITEM_QUALITY3_DESC.."|r",
	[ITEM_QUALITY_EPIC]= ITEM_QUALITY_COLORS[ITEM_QUALITY_EPIC].hex..ITEM_QUALITY4_DESC.."|r",
	[5]= ITEM_QUALITY_COLORS[5].hex..ITEM_QUALITY5_DESC.."|r",
	[6]= ITEM_QUALITY_COLORS[6].hex..ITEM_QUALITY6_DESC.."|r",
}

local GroupSelectionList = {
	["1"] = L["Group 1"],
	["2"] = L["Group 2"],
	["3"] = L["Group 3"],
	["4"] = L["Group 4"],
	["5"] = L["Group 5"],
	["6"] = L["Group 6"],
	["7"] = L["Group 7"],
	["8"] = L["Group 8"],
}

function HeadCount:OptionGet(info,key)
	if key ~= nil then
		return self.db.profile[info.arg][key]
	else
		return self.db.profile[info.arg]
	end
end

function HeadCount:OptionSetter(info,key,value)
	if value ~= nil then
		self.db.profile[info.arg][key] = value
	end
	self:UpdateConfig()
end

function HeadCount:UpdateConfig()
	self:GetModule("RaidTracker"):RAID_ROSTER_UPDATE(true)
	self:GetModule("RaidTracker"):manageWaitlistChange(self.db.profile.raid.waitlist.duration)
end

function HeadCount:AddBoss(info, input)
	local _, _, bossName = string.find(input, "^(%a+)$")
	local convertedBossName = input
	local validBoss = self:GetModule("BossDetection"):IsValidBossName(convertedBossName)
	if not validBoss and IsInInstance() then
		HeadCount:LogWarning(string.format(L["warning.boss.add.failure.missing"], HeadCount.TITLE, HeadCount.VERSION, convertedBossName))
		return
	end
	local raidTracker = HeadCount:GetModule("RaidTracker")
	if raidTracker then
		if raidTracker:isRaidActive() then
			local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
			local activityTime = HeadCount:NewInstance("Time",utcDateTimeInSeconds)
			local realZoneText = GetRealZoneText()
			local difficulty = HeadCount:determineDifficulty()
			local encounterName = convertedBossName
			local currentRaid = raidTracker:retrieveMostRecentRaid()
			raidTracker:processAttendance(activityTime)	-- update attendance
			local raidMemberSort = HeadCount:getRAID_MEMBER_SORT()
			local playerList = currentRaid:retrieveOrderedPlayerList("Name",
				HeadCount.db.profile.boss["groupsetup"]["raidlistgroup"], HeadCount.db.profile.boss["groupsetup"]["waitlistgroup"],	HeadCount.db.profile.boss["groupsetup"]["nolistgroup"],
				HeadCount.db.profile.boss["groupsetup"]["waitlist"], true, true)

			local bossargs = {
				["name"] = encounterName,
				["zone"] = realZoneText,
				["difficulty"] = difficulty,
				["runtime"] = activityTime,
				["players"] = playerList
			}

			local boss = HeadCount:NewInstance("Boss",bossargs)
			currentRaid:addBoss(boss)
			-- Broadcast the kill?
			if self.db.profile.isBossBroadcastEnabled then
				-- boss kill broadcasting is enabled
				local channel = self.db.profile.broadcastChannel
				HeadCount:SendMessage(string.format(L["guild.broadcast.manualbosskill"], encounterName), channel)
			end
			HeadCount:LogInformation(string.format(L["info.boss.add.success"], HeadCount.TITLE, HeadCount.VERSION, convertedBossName))
		else
			HeadCount:LogWarning(string.format(L["warning.boss.add.failure.activeraid"], HeadCount.TITLE, HeadCount.VERSION, convertedBossName))
		end
	else
		HeadCount:LogError(string.format(L["warning.boss.add.failure.raidtracker"], HeadCount.TITLE, HeadCount.VERSION))
	end
end
function HeadCount:AddLoot(info, input)
	local _, _, link, raidNumber, playerName = string.find(input, "^(|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r)%s+(%d+)%s+(%a+)$")	-- loot link
	local itemId = string.match(link, "item:(%d+):")
	local convertedRaidNumber = tonumber(raidNumber)
	local convertedPlayerName = HeadCount:convertStringToProperName(playerName)

	local raidTracker = HeadCount:GetModule("RaidTracker")
	if raidTracker then
		-- get the raid with ordered id of convertedraidnumber and add loot to that raid
		local orderedRaidList = raidTracker:retrieveOrderedRaidList(false)	-- ascending ordered raid list
		local raid = orderedRaidList[convertedRaidNumber]
		if (raid) then
			local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()
			local activityTime = HeadCount:NewInstance("Time",utcDateTimeInSeconds)
			local realZoneText = GetRealZoneText()
			local loot = HeadCount:NewInstance("Loot",{
				["itemId"] = itemId,
				["player"] = convertedPlayerName,
				["cost"] = 0,
				["looted"] = activityTime,
				["quantity"] = HeadCount.DEFAULT_LOOT_QUANTITY,
				["zone"] = realZoneText,
				["source"] = HeadCount.DEFAULT_LOOT_SOURCE,
				["note"] = nil,})

			raid:addLoot(loot)
			HeadCount:LogInformation(string.format(L["info.loot.add.success"], HeadCount.TITLE, HeadCount.VERSION, link, convertedPlayerName, convertedRaidNumber))
		else
			HeadCount:LogError(string.format(L["error.loot.add.failure.raidtracker"], HeadCount.TITLE, HeadCount.VERSION, convertedRaidNumber))
		end
	else
		HeadCount:LogError(string.format(L["warning.loot.add.failure.raidtracker"], HeadCount.TITLE, HeadCount.VERSION))
	end
end

-- configuration options
local options = {
    type = "group",
	handler = HeadCount,
	get = "OptionGet",
	set = "OptionSetter",
    args = {
		raid = {
			type = "group",
			name = L["raid.name"],
			desc = L["raid.description"],
			order = 2,
		    args = {
				groupsetup = {
					type = "group",
					name = L["groupsetup.name"],
					desc = L["groupsetup.description"],
					order = 1,
					args = {
						raidlistgroups = {
							type = "multiselect",
							name = L["raidlistgroups.name"],
							desc = L["raidlistgroups.description"],
							arg = "raidListGroups",
							values = GroupSelectionList,
							order = 1,
						},
						waitlistgroups = {
							type = "multiselect",
							name = L["waitlistgroups.name"],
							desc = L["waitlistgroups.description"],
							arg = "waitListGroups",
							values = GroupSelectionList,
							order = 2,
						},
						autogrouping = {
							type = "toggle",
							name = L["autogrouping.name"],
							desc = L["autogrouping.description"],
							get = function(info,key) return HeadCount.db.profile.raid.groupsetup[info.arg] 	end,
							set = function(info, value) HeadCount.db.profile.raid.groupsetup[info.arg] = value; HeadCount:UpdateConfig() end,
							arg = "isAutomaticGroupSelectionEnabled",
							width = "double",
							order = 3,
						},
					},
				},
				waitlist = {
					type = "group",
					name = L["waitlist.name"],
					desc = L["waitlist.description"],
					order = 2,
					get = function(info,key)
						return HeadCount.db.profile.raid.waitlist[info.arg]
					end,
					set = function(info, value)
						HeadCount.db.profile.raid.waitlist[info.arg] = value
						HeadCount:UpdateConfig()
					end,
					args = {
						duration = {
							type = "range",
							name = L["waitlist.duration.name"],
							desc = L["waitlist.duration.description"],
							arg = "duration",
							min = 0,
							max = 30,
							step = 1,
							order = 1,
						},
						autoremoval = {
							type = "toggle",
							name = L["waitlist.autoremoval.name"],
							desc = L["waitlist.autoremoval.description"],
							arg = "autoremoval",
							order = 2,
						},
						incoming = {
							type = "toggle",
							name = L["waitlist.incoming.name"],
							desc = L["waitlist.incoming.description"],
							arg = "incoming",
							width = "full",
							order = 3,
						},
						outgoing = {
							type = "toggle",
							name = L["waitlist.outgoing.name"],
							desc = L["waitlist.outgoing.description"],
							arg = "outgoing",
							width = "full",
							order = 4,
						},
						notify = {
							type = "toggle",
							name = L["waitlist.notify.name"],
							desc = L["waitlist.notify.description"],
							arg = "notify",
							width = "full",
							order = 5,
						}
					},
				},
				pruning = {
					type = "group",
					name = L["pruning.name"],
					desc = L["pruning.description"],
					order = 3,
					get = function(info,key)
						return HeadCount.db.profile.raid.pruning[info.arg]
					end,
					set = function(info, value)
						HeadCount.db.profile.raid.pruning[info.arg] = value
						HeadCount:UpdateConfig()
					end,
					args = {
						prune = {
							type = "toggle",
							name = L["pruning.prune.name"],
							desc = L["pruning.prune.description"],
							arg = "isPruningEnabled",
							order = 1,
							width = "full",
						},
						prunetime = {
							type = "range",
							name = L["prunetime.name"],
							desc = L["prunetime.description"],
							arg = "pruningTime",
							min = 1,
							max = 52,
							step = 1,
							order = 2,
							width = "double",
						},
					},
				},
				delay = {
					type = "range",
					name = L["delay.name"],
					desc = L["delay.description"],
					arg = "delay",
					get = function(info, key) return HeadCount.db.profile[info.arg] end,
					set = function(info, value) HeadCount.db.profile[info.arg] = value end,
					min = 0,
					max = 20,
					step = 1,
					order = 4,
				},
				bgtracking = {
					type = "toggle",
					arg = "isBattlegroundTrackingEnabled",
					name = L["bgtracking.name"],
					desc = L["bgtracking.description"],
					get = function(info, key) return HeadCount.db.profile[info.arg] end,
					set = function(info, value) HeadCount.db.profile[info.arg] = value end,
					order = 5,
				},
			},-- close Raid args Section
		},
		datetime = {
			type = "group",
			name = L["datetime.name"],
			desc = L["datetime.description"],
			order = 3,
			get = function(info, key) return HeadCount.db.profile[info.arg] end,
			set = function( info, value) HeadCount.db.profile[info.arg] = value end,
			args = {
			    dateformat = {
					type = "select",
					name = L["dateformat.name"],
					desc = L["dateformat.description"],
					arg = "dateFormat",
					values = HeadCount.DateFormats,
					order = 1,
				},
				timetotals = {
					type = "group",
					name = L["timetotals.name"],
					desc = L["timetotals.description"],
					order = 2,
					args = {
						raidlisttime = {
							type = "toggle",
							name = L["timetotals.raidlisttime.name"],
							desc = L["timetotals.raidlisttime.description"],
							arg = "isRaidListTimeEnabled",
							order = 1,
						},
						waitlisttime = {
							type = "toggle",
							name = L["timetotals.waitlisttime.name"],
							desc = L["timetotals.waitlisttime.description"],
							arg = "isWaitListTimeEnabled",
							order = 2,
						},
						offlinetime = {
							type = "toggle",
							name = L["timetotals.offlinetime.name"],
							desc = L["timetotals.offlinetime.description"],
							arg = "isOfflineTimeEnabled",
							order = 3,
						},
					},
				},
				timezone = {
					type = "range",
					name = L["timezone.name"],
					desc = L["timezone.description"],
					arg = "timezone",
					min = -10 ,
					max = 14,
					step = 1,
					order = 3,
				},
				timeformat = {
					type = "toggle",
					name = L["timeformat.name"],
					desc = L["timeformat.description"],
					arg = "isTimeFormatEnabled",
					order = 4,
				},
			},
		},
		boss = {
			type = "group",
			name = L["boss.name"],
			desc = L["boss.description"],
			order = 4,
			args = {
				["add"] = {
					type = "input",
					name = L["boss.add.name"],
					desc = L["boss.add.description"],
					get = false,
					set = "AddBoss",
					order = 1,
				},
				groupsetup = {
					type = "group",
					get = function(info,key)
						return HeadCount.db.profile.boss.groupsetup[info.arg]
					end,
					set = function(info, value)
						HeadCount.db.profile.boss.groupsetup[info.arg] = value
						HeadCount:UpdateConfig()
					end,
					name = L["boss.groupsetup.name"],
					desc = L["boss.groupsetup.description"],
					order = 2,
					args = {
						raidlistgroup = {
							type = "toggle",
							name = L["boss.groupsetup.raidlistgroup.name"],
							desc = L["boss.groupsetup.raidlistgroup.description"],
							arg = "raidlistgroup",
							order = 1,
						},
						waitlistgroup = {
							type = "toggle",
							name = L["boss.groupsetup.waitlistgroup.name"],
							desc = L["boss.groupsetup.waitlistgroup.description"],
							arg = "waitlistgroup",
							order = 2,
						},
						nolistgroup = {
							type = "toggle",
							name = L["boss.groupsetup.nolistgroup.name"],
							desc = L["boss.groupsetup.nolistgroup.description"],
							arg = "nolistgroup",
							order = 3,
						},
						waitlist = {
							type = "toggle",
							name = L["boss.groupsetup.waitlist.name"],
							desc = L["boss.groupsetup.waitlist.description"],
							arg = "waitlist",
							order = 4,
						},
					},
				},
			},
		},
		loot = {
				type = "group",
				get = function(info, key) return HeadCount.db.profile[info.arg] end,
				set = function( info, value) HeadCount.db.profile[info.arg] = value end,
				name = L["loot.name"],
				desc = L["loot.description"],
				order = 5,
			    args = {
					minimum = {
						type = "select",
						name = L["loot.minimum.name"],
						desc = L["loot.minimum.description"],
						arg = "minimumLootQuality",
						values = lootQualityList,
						order = 1,
					},
					popup = {
						type = "toggle",
						name = L["loot.popup.name"],
						desc = L["loot.popup.description"],
						arg = "isLootPopupEnabled",
						order = 2,
					},
					exclude = {
						type = "description",
						name = L["loot.exclude.description"],
						order = 4,
						width = "full",
					},
					addLoot = {
						type = "input",
						name = L["loot.add.name"],
						desc = L["loot.add.description"],
						get = false,
						set = "AddLoot",
						width = "full",
						order = 3,
					},
				},
			},
			export = {
				type = "group",
				name = L["export.name"],
				desc = L["export.description"],
				order = 6,
				args = {
					format = {
						type = "select",
						name = L["export.format.name"],
						desc = L["export.format.description"],
						get = function(info, key) return HeadCount.db.profile[info.arg] end,
						set = function( info, value) HeadCount.db.profile[info.arg] = value end,
						arg = "exportFormat",
						values = exportFormats,
						order = 1,
					},
					eqdkp = {
						type = "group",
						name = L["export.eqdkp.name"],
						desc = L["export.eqdkp.description"],
						order = 2,
						args = {
							difficulty = {
								type = "toggle",
								get = function(info, key) return HeadCount.db.profile.export.eqdkp.difficulty end,
								set = function( info, value) HeadCount.db.profile.export.eqdkp.difficulty = value end,
								name = L["export.eqdkp.difficulty.name"],
								desc = L["export.eqdkp.difficulty.description"],
								order = 1,
								arg = "difficulty",
						},
					},
				},
			},
		},
		reporting = {
			type = "group",
			name = L["reporting.name"],
			desc = L["reporting.description"],
			get = function(info, key) return HeadCount.db.profile[info.arg] end,
			set = function( info, value) HeadCount.db.profile[info.arg] = value end,
			order = 7,
			args = {
				broadcastchannel = {
					type = "select",
					name = L["reporting.broadcastchannel.name"],
					desc = L["reporting.broadcastchannel.description"],
					arg = "broadcastChannel",
					values = channels,
					order = 1,
				},
				spacer = {
					type = "description",
					name = "",
					order = 2,
				},
				bosskills = {
					type = "toggle",
					name = L["reporting.bosskills.name"],
					desc = L["reporting.bosskills.description"],
					arg = "isBossBroadcastEnabled",
					order = 3,
				},
				loot = {
					type = "toggle",
					name = L["reporting.loot.name"],
					desc = L["reporting.loot.description"],
					arg = "isLootBroadcastEnabled",
					order = 4,
				},
			},
		},
		exclude = {
			type = "group",
			name = L["exclude.name"],
			desc = L["exclude.description"],
			order = 9,
			args = {
				add = {
					type = "input",
					name = L["exclude.add.name"],
					desc = L["exclude.add.description"],
					get = false,
					set = function(info, value)
						local id = string.match(value, "item:(%d+):")
						if not id then
							id = value
						end
						local numberId = tonumber(id)
						HeadCount.db.profile.exclusionList[numberId] = true
						HeadCount:LogInformation(string.format(L["info.exclude.add.success"], HeadCount.TITLE, HeadCount.VERSION, value))
					end,
					width = "full",
					order = 2,
				},
				remove = {
					type = "input",
					name = L["exclude.remove.name"],
					desc = L["exclude.remove.description"],
					get = false,
					set = function(info, link)
						local id = string.match(link, "item:(%d+):")
						if not id then
							id = link
						end
						local numberId = tonumber(id)
						HeadCount.db.profile.exclusionList[numberId] = nil
						HeadCount:LogInformation(string.format(L["info.exclude.remove.success"], HeadCount.TITLE, HeadCount.VERSION, link))
					end,
					width = "full",
					order = 3,
				},
		        list = {
		            type = "multiselect",
		            name = L["exclude.list.name"],
					desc = L["exclude.list.description"],
					get = function(info, key) return HeadCount.db.profile.exclusionList[key] end,
					set = function(info, key, value) HeadCount.db.profile.exclusionList[key] = value end,
					values = function()
						-- Build Select List
						local list = {}
						for k,v in pairs(HeadCount.db.profile.exclusionList) do
							local itemName, itemLink, _, _, _, _, _, _, _, texture = GetItemInfo(k)
							if not itemLink then
								HeadCount:LogDebug(string.format(L["info.exclude.not.in.cache"],HeadCount.TITLE, HeadCount.VERSION, k))
								list[k] = string.format(L["info.exclude.missing"],k)
							else
								list[k] = itemLink --string.format(L["info.exclude.link"],itemLink,k)
							end
						end
						return list
					end,
					width="fill",
					order = 1,
		        },
			},
		},
		debug = {
			type = "toggle",
			name = L["debug.name"],
			desc = L["debug.description"],
			arg = "isDebugEnabled",
			get = function(info, key) return HeadCount.db.profile[info.arg] end,
			set = function(info, value) HeadCount.db.profile[info.arg] = value end,
			order = 8,
		},
	},
} -- Close Option

local defaults = {
	-- Group Management
	raidListGroups = {
		["1"] = true,
		["2"] = true,
		["3"] = true,
		["4"] = true,
		["5"] = true,
		["6"] = false,
		["7"] = false,
		["8"] = false,
	},
	waitListGroups = {
		["1"] = false,
		["2"] = false,
		["3"] = false,
		["4"] = false,
		["5"] = false,
		["6"] = true,
		["7"] = true,
		["8"] = true,
	},
	-- raid management
	raid = {
		["groupsetup"] = {
			["isAutomaticGroupSelectionEnabled"] = true,
		},
		["waitlist"] = {
			["duration"] = 0, 		-- default announce duration of always enabled
			["autoremoval"] = true, -- automatically remove players from the wait list upon joining a raid list group
			["incoming"] = true, 	-- display incoming wait list whispers
			["outgoing"] = false, 	-- do NOT display outgoing wait list whispers
			["notify"] = false, 	-- do NOT display the waitlist notification message
		},
		["pruning"] = {
			["isPruningEnabled"] = false,
			["pruningTime"] = 12,	-- default pruning time of ~3 months
		},
	},
	-- boss management
	boss = {
		["groupsetup"] = {
			["raidlistgroup"] = true,
			["waitlistgroup"] = false,
			["nolistgroup"] = false,
			["waitlist"] = false,
		},
	},
	delay = 5,	-- default delay value of 5 seconds
	isBattlegroundTrackingEnabled = false,
	dateFormat = 2,
	timezone = -5, -- "-5 GMT, Eastern Standard Time"
	isTimeFormatEnabled = true,
	isRaidListTimeEnabled = true,
	isWaitListTimeEnabled = false,
	isOfflineTimeEnabled = false,
	minimumLootQuality = 4,
	isLootPopupEnabled = false,
	broadcastChannel = "GUILD",
	isLootBroadcastEnabled = false,
	isBossBroadcastEnabled = false,
	exportFormat = "EQdkp",
	-- export
	export = {
		["eqdkp"] = {
			["difficulty"] = true,
		},
	},
	isDebugEnabled = false,
	raidListWrapper = nil,
	exclusionList = {
		[20725] = true, -- Nexus Crystal
		[22450] = true, -- Void Crystal
		[30311] = true, -- Warp Slicer
		[30312] = true, -- Infinity Blade
		[30313] = true, -- Staff of Disintegration
		[30314] = true, -- Phaseshift Bulwark
		[30316] = true, -- Devastation
		[30317] = true, -- Cosmic Infuser
		[30318] = true, -- Netherstrand Longbow
		[34057] = true, -- Abyss Crystal
		[36919] = true, -- Cardinal Ruby
		[36922] = true, -- King's Amber
		[36925] = true, -- Majestic Zircon
		[36928] = true, -- Dreadstone
		[36931] = true, -- Ametrine
		[36934] = true, -- Eye of Zul
		[49110] = true, -- Nightmare Tear
	},
}

function HeadCount:GetDBDefaults()
	return { profile = defaults }
end

function HeadCount:GetExportFormat()
	return self.db.profile.exportFormat
end

function HeadCount:SetupOptions()
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(HeadCount.db)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("HeadCount2", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HeadCount2", "HeadCount2")
	self:RegisterChatCommand("hc2", function() LibStub("AceConfigDialog-3.0"):Open("HeadCount2") end )
end
