local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
local L = LibStub("AceLocale-3.0"):GetLocale("HeadCount2")

local Fields = {
	name = UNKNOWN,
	className = UNKNOWN,
	fileName = UNKNOWN,
	race = UNKNOWN,
	guild = nil,
	sex = 1,
	level = 0,
	isPresentInRaidList = false,	-- raid list raid group
	isPresentInWaitList = false,	-- wait list raid group
	isOnline = false,
	isWaitlisted = false,			-- artificial wait list
	waitlistActivityTime = nil,	-- artificial wait list activity time
	waitlistNote = "",			-- artificial wait list note
	raidListTime = 0,
	waitListTime = 0,
	offlineTime = 0,
	timeList = {},
	lastActivityTime = nil,
	isFinalized = false,
}

local Player = HeadCount:NewClassFunctions()

function Player:Init(args)
	if args then
		print("Player setup started")
		self.name = args["name"]
		self.className = args["className"]
		self.fileName = args["fileName"]
		self.race = args["race"]
		self.guild = args["guild"]
		self.sex = args["sex"]
		self.level = args["level"]
		self.isPresentInRaidList = args["isPresentInRaidList"]
		self.isPresentInWaitList = args["isPresentInWaitList"]
		self.isOnline = args["isOnline"]
		self.isWaitlisted = args["isWaitlisted"]
		self.waitlistActivityTime = args["waitlistActivityTime"]
		self.waitlistNote = args["waitlistNote"]
		self.raidListTime = args["raidListTime"]
		self.waitListTime = args["waitListTime"]
		self.offlineTime = args["offlineTime"]
		self.timeList = args["timeList"]
		self.lastActivityTime = args["lastActivityTime"]
		self.isFinalized = args["isFinalized"]
	end
end
-- Gets the real sex.
-- 1 = Unknown
-- 2 = Male
-- 3 = Female
-- @return string Returns the real sex.
function Player:getRealSex()
	local realSex = nil
	if (2 == self.sex) then
		realSex = MALE
	elseif (3 == self.sex) then
		realSex = FEMALE
	else
		realSex = UNKNOWN
	end
	return realSex
end
function Player:processBeginTime(activityTime, pnote)
	local numberOfPlayerTimePairs = #self.timeList	-- get the total number of current time pairs
	if (activityTime and numberOfPlayerTimePairs > 0) then
		local lastBeginTime = self.timeList[numberOfPlayerTimePairs].start
		local lastEndTime = self.timeList[numberOfPlayerTimePairs].stop
		if ((lastBeginTime) and (lastEndTime)) then
			-- both time values exist for the most recent pair, start a new pair
			local timePair = HeadCount:NewInstance("TimePair",{start = activityTime, note = pnote, stop = 0})
			table.insert(self.timeList, timePair)
		elseif ((not lastBeginTime) and (not lastEndTime)) then
			-- most recent pair contains no time values
			self.timeList[numberOfPlayerTimePairs].start = activityTime
			self.timeList[numberOfPlayerTimePairs].note = pnote
		end
	end
end

-- Processes an end time
-- @param activityTime The activity time.
function Player:processEndTime(activityTime)
	local numberOfPlayerTimePairs = #self.timeList	-- get the total number of current time pairs
	if (activityTime and numberOfPlayerTimePairs > 0) then
		local lastBeginTime = self.timeList[numberOfPlayerTimePairs].start
		local lastEndTime = self.timeList[numberOfPlayerTimePairs].stop
		if ((lastBeginTime) and (not lastEndTime)) then
			-- the most recent beginning time exists
			-- no recent end time exists, add it
			self.timeList[numberOfPlayerTimePairs].stop = activityTime
		end
	end
end
-- Moves a player to the raid list.
-- @param activityTime The activity time.
-- @return boolean Returns true if the player was moved, false otherwise
function Player:moveToRaidList(activityTime)
	local isChanged = false
	local timeDifference = HeadCount:computeTimeDifference(activityTime, self.lastActivityTime)
	local isWaitlistAutoremovalEnabled = HeadCount.db.profile.raid.waitlist.autoremoval

	if (self.isPresentInWaitList) then
		-- player was in the wait list, now in the raid list
		self.isPresentInRaidList = true
		self.isPresentInWaitList = false
		self.waitListTime = self.waitListTime + timeDifference
		self:processEndTime(activityTime)
		self:processBeginTime(activityTime, L["Raid list"])
		self.lastActivityTime = activityTime
		self.isFinalized = false
		isChanged = true
	elseif (not (self.isPresentInRaidList or self.isPresentInWaitList)) then
		-- player was in no list, now in the raid list
		self.isPresentInRaidList = true
		self.isPresentInWaitList = false
		if (not self.isOnline) then
			-- player was offline
			self.offlineTime = self.offlineTime + timeDifference
			self.isOnline = true
		end

		self:processEndTime(activityTime)
		self:processBeginTime(activityTime, L["Raid list"])
		self.lastActivityTime =activityTime
		self.isFinalized = false
		isChanged = true
	end
	-- do nothing if player is in the raid list (no change) or both lists (error)
	if (isWaitlistAutoremovalEnabled) then
		self.isWaitlisted =false
		self.waitlistActivityTime = nil
	end
	return isChanged
end
function Player:moveToWaitList(activityTime)
	local isChanged = false
	local timeDifference = HeadCount:computeTimeDifference(activityTime, self.lastActivityTime)

	self.isWaitlisted =true			-- wait list group players are automatically on wait list
	self.waitlistActivityTime = activityTime

	if (self.isPresentInRaidList) then
		-- player was in the raid list, now in the wait list
		self.isPresentInWaitList = true	-- player is in the wait list group
		self.isPresentInRaidList = false
		self.raidListTime = self.raidListTime + timeDifference
		self:processEndTime(activityTime)
		self:processBeginTime(activityTime, L["Wait list"])
		self.lastActivityTime = activityTime
		self.isFinalized = false
		isChanged = true
	elseif (not (self.isPresentInRaidList or self.isPresentInWaitList)) then
		-- player was in no list, now in the wait list
		self.isPresentInWaitList = true	-- player is in the wait list group
		self.isPresentInRaidList =false
		if (not self.isOnline) then
			-- player was offline
			self.offlineTime = self.offlineTime + timeDifference
			self.isOnline = true
		end
		self:processEndTime(activityTime)
		self:processBeginTime(activityTime, L["Wait list"])
		self.lastActivityTime = activityTime
		self.isFinalized = false
		isChanged = true
	end
	-- do nothing if player is in the wait list (no change) or both lists (error)
	return isChanged
end
function Player:moveToNoList(activityTime, isOnline)
	local isChanged = false
	local timeDifference = HeadCount:computeTimeDifference(activityTime, self.lastActivityTime)

	if (isOnline) then
		-- player is currently online
		if (self.isPresentInRaidList) then
			self.isPresentInRaidList = false
			self.isPresentInWaitList = false
			self.raidListTime = self.raidListTime + timeDifference
			self:processEndTime(activityTime)
			self:processBeginTime(activityTime, L["No list"])
			self.lastActivityTime = activityTime
			self.isFinalized = false
			isChanged = true
		elseif (self.isPresentInWaitList) then
			self.isPresentInRaidList = false
			self.isPresentInWaitList = false
			self.waitListTime = self.waitListTime + timeDifference
			self:processEndTime(activityTime)
			self:processBeginTime(activityTime, L["No list"])
			self.lastActivityTime = activityTime
			self.isFinalized = false
			isChanged = true
		elseif (not (self.isPresentInRaidList or self.isPresentInWaitList)) then
			if (not self.isOnline) then
				-- player was offline, now is online
				self.isPresentInRaidList = false
				self.isPresentInWaitList = false
				self.offlineTime = self.offlineTime + timeDifference
				self.isOnline = true
				self:processEndTime(activityTime)
				self:processBeginTime(activityTime, L["No list"])
				self.lastActivityTime = activityTime
				self.isFinalized = false
				isChanged = true
			end
		end
	else
		-- player is currently offline
		if (self.isOnline) then
			-- player was online, now is offline
			if (self.isPresentInRaidList) then
				self.isPresentInRaidList = false
				self.isPresentInWaitList = false
				self.raidListTime = self.raidListTime + timeDifference
				self.isOnline = false
				self:processEndTime(activityTime)
				self:processBeginTime(activityTime, L["Offline"])
				self.lastActivityTime = activityTime
				self.isFinalized = false
				isChanged = true
			elseif (self.isPresentInWaitList) then
				self.isPresentInRaidList = false
				self.isPresentInWaitList = false
				self.waitListTime = self.waitListTime + timeDifference
				self.isOnline = false
				self:processEndTime(activityTime)
				self:processBeginTime(activityTime, L["Offline"])
				self.lastActivityTime = activityTime
				self.isFinalized = false
				isChanged = true
			end
		end
	end
	return isChanged
end
function Player:updateValues(position, className, fileName)
	if (position) then
		if (not self.className or self.className == UNKNOWN ) then
			self.className = className
		end
		if (not self.fileName or self.fileName == UNKNOWN) then
			self.fileName = fileName
		end
		if (0 == self.level) then
			-- level is not set
			self.level = UnitLevel("raid" .. position)
		end
		if (not self.race or self.race == UNKNOWN) then
			local _, race = UnitRace("raid" .. position)
			self.race = race
		end
		if (not self.guild) then
			-- guild is not set
			self.guild = GetGuildInfo("raid" .. position)
		end
		if (self.sex == 1) then
			self.sex = UnitSex("raid" .. position)
		end
	end
end
-- Gets the begin time for the player.
-- @return number Returns the player begin time
function Player:retrieveStartingTime()
	return self.timeList[1].start
end
-- Gets the end time for the player.
-- @return number Returns the player begin time
function Player:retrieveEndingTime()
	local numberOfTimes = #self.timeList
	return self.timeList[numberOfTimes].stop
end
function Player:getTotalTime()
	local isRaidListTimeEnabled = HeadCount.db.profile.isRaidListTimeEnabled
	local isWaitListTimeEnabled = HeadCount.db.profile.isWaitListTimeEnabled
	local isOfflineTimeEnabled  = HeadCount.db.profile.isOfflineTimeEnabled
	local totalTime = 0
	if (isRaidListTimeEnabled) then
		totalTime = totalTime + self.raidListTime
	end
	if (isWaitListTimeEnabled) then
		totalTime = totalTime + self.waitListTime
	end
	if (isOfflineTimeEnabled) then
		totalTime = totalTime + self.offlineTime
	end
	return totalTime
end
function Player:finalize(currentTime)
	if (not self.isFinalized) then
		local numberOfPlayerTimePairs = # self.timeList
		local lastBeginTime = self.timeList[numberOfPlayerTimePairs].start
		local lastEndTime = self.timeList[numberOfPlayerTimePairs].stop
		if (not lastEndTime) then
			-- ending player time is not yet set
			self.timeList[numberOfPlayerTimePairs].stop = currentTime
			local timeDifference = HeadCount:computeTimeDifference(currentTime, lastBeginTime)
			if (self.isOnline) then
				-- player is online
				if (self.isPresentInRaidList) then
					-- player is online and in raid list
					self.raidListTime = self.raidListTime + timeDifference
				elseif (self.isPresentInWaitList) then
					-- player is online and in raid list
					self.waitListTime = self.waitListTime + timeDifference
				end
			else
				-- player is offline
				self.offlineTime = self.offlineTime + timeDifference
			end
		end
		self.isPresentInRaidList = false
		self.isPresentInWaitList = false
		self.isOnline = true
		self.lastActivityTime = currentTime
		self.isFinalized = true
	end
	return self.isFinalized
end
function Player:recover(activityTime, lastActivityTime)
	local timeIndex = # self.timeList
	local lastBeginTime = self.timeList[timeIndex].start
	local lastEndTime = self.timeList[timeIndex].stop
	if ((lastBeginTime) and (not lastEndTime)) then
		local timeDifference = HeadCount:computeTimeDifference(lastActivityTime, lastBeginTime)
		if (self.isPresentInRaidList) then
			-- player is in the raid list
			self.raidListTime = self.raidListTime + timeDifference
			self:processEndTime(lastActivityTime)
			self:processBeginTime(activityTime, L["Raid list"] .. " - " .. L["Recovery"])
		elseif (self.isPresentInWaitList) then
			-- player is in the wait list
			self.waitListTime = self.waitListTime + timeDifference
			self:processEndTime(lastActivityTime)
			self:processBeginTime(activityTime, L["Wait list"] .. " - " .. L["Recovery"])
		else
			-- player is in no list
			self:processEndTime(lastActivityTime)
			if (self.isOnline) then
				-- player is online
				self:processBeginTime(activityTime, L["No list"] .. " - " .. L["Recovery"])
			else
				-- player is offline
				self.offlineTime = self.offlineTime + timeDifference
				self:processBeginTime(activityTime, L["Offline"] .. " - " .. L["Recovery"])
			end
		end
	end
	self.lastActivityTime =activityTime
	self.isFinalized = false
end
-- Add the Class Definition to Registry
HeadCount:DefineClass("Player",Fields,Player)