local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")

HeadCount.__ClassRegistry = {}

local function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end


function HeadCount:DefineClass(name,fields,functions)
	if not HeadCount.__ClassRegistry[name] then
		fields.__ClassName = name
		local fc = deepcopy(fields)
		HeadCount.__ClassRegistry[name] = {
			Fields = fc,
			Functions = {__index = functions }
		}
	end
end

function HeadCount:NewClassFunctions()
	local class = {}
	function class:toString()
		return self.__ClassName
	end
	return class
end

local function __AddMetaData(tbl)
	HeadCount:AddClassMeta(tbl)
	local tblClass = tbl.__ClassName
	if HeadCount.__ClassRegistry[tblClass] then
		setmetatable(tbl,HeadCount.__ClassRegistry[tblClass].Functions)
	end
end

function HeadCount:AddClassMeta(data)
	local tblClass = data.__ClassName
	if HeadCount.__ClassRegistry[tblClass] then
		setmetatable(data,HeadCount.__ClassRegistry[tblClass].Functions)
	end
	for k,v in pairs(data) do
		if type(v) == "table" then
			__AddMetaData(v)
		end
	end
end

function HeadCount:NewInstance(name,args)
	assert(HeadCount.__ClassRegistry[name],"No such class exists")
	local fields = deepcopy(HeadCount.__ClassRegistry[name].Fields)
	local func = HeadCount.__ClassRegistry[name].Functions
	setmetatable(fields,func)
	if args and fields.Init then
		fields:Init(args)
	end
	return fields
end